from appJar import gui
import os
import time
#import thread
#from threading import Thread
from opt_parser import manage_local, manage_conf_update, manage_show_conf


class Gui:
    def __init__(self):
        self.app = gui(handleArgs=False)
        self.configure_main_window()
        self.create_progress_subwindow()
        #self.p_bar_thread = Thread(target=self.loop_progress,)
        #self.p_bar_thread.start()
        self.app.go()

    def button_manager_main(self, button):
        if (button == "Validate file"):
            path = self.app.getEntry("file_path")
            manage_conf_update(None, path)
        elif(button == "Show configuration"):
            manage_show_conf(None, no_check=True)
            self.app.infoBox("Take a look Closer", "Configuration file status displayed in console.")
        if (button == "Tar Me"):
            description_file_path = self.get_description()
            self.app.showSubWindow("sub1")
            manage_local(None, no_check=True, description_file=description_file_path)
            self.app.hideSubWindow("sub1")
            self.app.infoBox("success", "Success Fully created Tarball, Read console log for more details")
            self.app.clearTextArea("pb_describe")
            self.app.setTextArea("pb_describe", "Describe your problem")

    def create_progress_subwindow(self):
        self.app.startSubWindow("sub1", title="Please Wait...", modal=False, transient=True, blocking=False)
        self.app.addDualMeter("progress")
        self.app.setGeometry(300, 100)
        self.app.stopSubWindow()

    def loop_progress(self):
        while self.app:
            for i in range(0,100):
                print self.app.getMeter("progress")
                self.app.setMeter("progress", [i, i])
                time.sleep(0.1)

    def configure_main_window(self):
        self.app.addTextArea("pb_describe")
        self.app.setTextArea("pb_describe", "Describe your problem")
        self.app.addButtons(["Show configuration", "Tar Me"], self.button_manager_main)
        self.app.addFileEntry("file_path")
        self.app.addButton("Validate file", self.button_manager_main)

    def get_description(self):
        input = self.app.getTextArea("pb_describe")
        if(input != "Describe your problem"):#here we should save in file
            description_file_name = "bug describer"
            fp = open(description_file_name, "w")
            fp.write(input)
            fp.close()
            cwd = os.getcwd()
            file_path = cwd + "/" + description_file_name
            return file_path
        return None



import colors
import os
from os.path import expanduser, isdir
from os import listdir
import re


def manage_entries(entry):
    """
    Some entries in the configuration file are not path but keywords, this function translate those keywords to
    path,
    :param entry: Value from the configuration file
    :return: A list of paths "calculated" from entries found in the configuration file .log_to_nurc file
    """
    file_list = []
    robot_env_patten = re.compile('.*/.gopher_robot_environment.bash$')
    if (entry == 'latest_log'):
        file_list.append(manage_ros_log())
        return file_list
    elif(robot_env_patten.match(entry)):
        return  get_semantics_from_robot_env(entry)

    file_list.append(entry)
    return file_list

def manage_ros_log():
    """
    As .ros/log/latest symlink is broken we manually search the path to the last ros log.

    :return: The path of the latest log folder.
    """
    log_folder_path = expanduser("~/.ros/log")
    try:
        log_folder_content = listdir(log_folder_path)
    except OSError:
        print (colors.bcolors.FAIL + "Ros log folder %s does not exist..." % (log_folder_path) + colors.bcolors.ENDC)
        exit(1)
    log_folder_list = [] #list of folders under ~/.ros/log
    for element in log_folder_content:
        actual_folder_path = log_folder_path + "/" + element
        if (isdir(actual_folder_path)):
            log_folder_list.append((actual_folder_path,element))

    #remove folder not matching the xxxx-xx-xx pattern
    exact_log_folder_list = []
    pattern = re.compile("[0-9]{4}-{1}[0-9]{2}-{1}[0-9]{2}")
    for element in log_folder_list:
        if pattern.match(element[1]):
            exact_log_folder_list.append(element)
    log_folder_list = exact_log_folder_list

    sorted_log_folder_list = sorted(log_folder_list, key=lambda tup: tup[1])

    latest = None
    try :
        latest = ((sorted_log_folder_list[-1])[0])
    except IndexError:
        print (colors.bcolors.FAIL + "No log folder has been found une ~/.ros/log" + colors.bcolors.ENDC)

    return latest + "/latest"

def get_semantics_from_robot_env(robot_env_file_path):
    """
    Grab from the robot configuration file the semantic file path.
    :return: The semantic_file_paht if it's found.
    """
    to_ret = []
    to_ret.append(robot_env_file_path)
    robot_env_file_path = expanduser(robot_env_file_path)

    if os.path.isfile(robot_env_file_path):
        robot_env_fp = open(robot_env_file_path)

        line = robot_env_fp.readline()
        pattern = re.compile("export GOPHER_SEMANTICS=[^=]+")

        while line:
            line_list = line.split('#')
            line = line_list[0] # we grab here everything before the fisrt comment
            if line and pattern.match(line):
                line = (line.split("="))[1] #We get what is on the right side of the equal, Now line is the path
                line = line.strip()
                line = expanduser(line)     #of the semantic file... we should still check existence OFC
                if os.path.isfile(line):
                    to_ret.append(line)
                    return to_ret  #path to semantics
                else:
                    print (colors.bcolors.WARNING + "Path provided in .gopher_robot_environement.bash (%s) is not a "
                                                    "valid absolute or relative path. Semantic file won't be tared"
                                                    % (line) + colors.bcolors.ENDC)

            line = robot_env_fp.readline()
        robot_env_fp.close()

    return to_ret


from cStringIO import StringIO
import Tkinter
import opt_parser
import os
import sys
import thread
import time
import colors

def disable_colors():
    colors.bcolors.HEADER = ''
    colors.bcolors.OKBLUE = ''
    colors.bcolors.OKGREEN = ''
    colors.bcolors.WARNING = ''
    colors.bcolors.FAIL = ''
    colors.bcolors.ENDC = ''
    colors.bcolors.BOLD = ''
    colors.bcolors.UNDERLINE = ''


def callback(text_zone, btn_name):
    description_file = get_text(text_zone)
    if (btn_name == "local"):
        thread.start_new_thread(opt_parser.manage_local,(None, True, description_file))
    if (btn_name == "server"):
        thread.start_new_thread(opt_parser.manage_send,(None, True, description_file))


def make_gui():
    disable_colors()
    sys.stdout = std_out_buffer = StringIO()
    main_window = Tkinter.Tk()
    to_pack_list = []

    text_box = generate_text_zone(main_window)
    text_box.pack()

    console = generate_console(main_window)
    console.pack()

    to_pack_list.append(generate_btn(main_window, "Save locally", callback, text_box, "local"))
    to_pack_list.append(generate_btn(main_window, "Send to server", callback, text_box, "server"))

    for element in to_pack_list :
        element.pack()

    while main_window:
        main_window.update_idletasks()
        main_window.update()
        update_console(std_out_buffer, console)
        time.sleep(0.1)


def generate_console(main_window):
    console = Tkinter.Text(main_window, height=10, width=60, background="black", fg="white", font=("Helvetica", 14))
    return console

def update_console(console_stream, console):
    str = console_stream.getvalue()
    console_stream.truncate(0)
    if str:
        console.insert(Tkinter.END, str)
        console.yview_pickplace("end")


def generate_text_zone(main_window):
    text_zone = Tkinter.Text(main_window, height=5, width=60, font=("Helvetica", 16))
    text_zone.pack()
    text_zone.insert(Tkinter.END, "Describe your problem")
    return text_zone

def generate_btn(main_window, btn_text, callback_func, callback_arg1, callback_arg2):
    btn = Tkinter.Button(main_window, text=btn_text, command=lambda: callback_func(callback_arg1, callback_arg2 ))
    return btn

def get_text(text_box):
    input = text_box.get("1.0", Tkinter.END)
    if(input != "Describe your problem\n"):#here we should save in file
        description_file_name = "bug describer"
        fp = open(description_file_name, "w")
        fp.write(input)
        fp.close()
        cwd = os.getcwd()
        file_path = cwd + "/" + description_file_name
        return file_path
    return None

make_gui()
